<?php

namespace WPDesk\ShopMagic\Event;

use WPDesk\ShopMagic\Automation\Automation;
use WPDesk\ShopMagic\Event\Builtin\CustomerAccountCreated;
use WPDesk\ShopMagic\Event\Builtin\CustomerOptedIn;
use WPDesk\ShopMagic\Event\Builtin\CustomerOptedOut;
use WPDesk\ShopMagic\Event\Builtin\OrderCancelled;
use WPDesk\ShopMagic\Event\Builtin\OrderCompleted;
use WPDesk\ShopMagic\Event\Builtin\OrderFailed;
use WPDesk\ShopMagic\Event\Builtin\OrderManualTrigger;
use WPDesk\ShopMagic\Event\Builtin\OrderNew;
use WPDesk\ShopMagic\Event\Builtin\OrderOnHold;
use WPDesk\ShopMagic\Event\Builtin\OrderPending;
use WPDesk\ShopMagic\Event\Builtin\OrderProcessing;
use WPDesk\ShopMagic\Event\Builtin\OrderRefunded;
use WPDesk\ShopMagic\Event\Builtin\OrderStatusChanged;
use WPDesk\ShopMagic\Event\DeferredStateCheck\OrderStatusDeferredEvent;
use WPDesk\ShopMagic\Filter\FilterLogic;

final class EventFactory {
	const GROUP_USERS = 'users';
	const GROUP_ORDERS = 'orders';

	/** @return Event */
	public function create_event( $slug, Automation $automation, FilterLogic $filters ) {
		$event = clone $this->get_event( $slug );
		$event->set_automation( $automation );
		$event->set_filter_logic( $filters );

		return $event;
	}

	/**
	 * @return |Event[]
	 */
	public function get_event_list() {
		return apply_filters( 'shopmagic/core/events', $this->get_build_in_events() );
	}

	/**
	 * @param string $group_id
	 *
	 * @return string
	 */
	public function event_group_name( $group_id ) {
		$groups = apply_filters( 'shopmagic/core/groups', [
			self::GROUP_ORDERS => __( 'Orders', 'shopmagic-for-woocommerce' ),
			self::GROUP_USERS  => __( 'Customers', 'shopmagic-for-woocommerce' )
		] );

		return $groups[ $group_id ] ?: '';
	}

	/**
	 * @param string
	 *
	 * @return Event
	 */
	public function get_event( $slug ) {
		$events = $this->get_event_list();
		if ( isset( $events[ $slug ] ) ) {
			return apply_filters( 'shopmagic/core/single_event', $events[ $slug ] );
		}

		return new NullEvent();
	}

	/**
	 * @return Event[]
	 */
	private function get_build_in_events() {
		$events = array(
			'shopmagic_order_new_event'        => new OrderNew(),
			'shopmagic_order_pending_event'    => new OrderStatusDeferredEvent( new OrderPending(), 'pending' ),
			'shopmagic_order_processing_event' => new OrderStatusDeferredEvent( new OrderProcessing(), 'processing' ),
			'shopmagic_order_cancelled_event'  => new OrderStatusDeferredEvent( new OrderCancelled(), 'cancelled' ),
			'shopmagic_order_completed_event'  => new OrderStatusDeferredEvent( new OrderCompleted(), 'completed' ),
			'shopmagic_order_failed_event'     => new OrderStatusDeferredEvent( new OrderFailed(), 'failed' ),
			'shopmagic_order_on_hold_event'    => new OrderStatusDeferredEvent( new OrderOnHold(), 'on-hold' ),
			'shopmagic_order_refunded_event'   => new OrderStatusDeferredEvent( new OrderRefunded(), 'refunded' ),
			'shopmagic_order_status_changed'   => new OrderStatusChanged(),

			'shopmagic_new_account_event'     => new CustomerAccountCreated(),
			'shopmagic_customer_optin_event'  => new CustomerOptedIn(),
			'shopmagic_customer_optout_event' => new CustomerOptedOut()
		);

		return $events;
	}
}
