<?php

namespace WPDesk\ShopMagic\Event;

use WPDesk\ShopMagic\Exception\ReferenceNoLongerAvailableException;

abstract class UserCommonEvent extends BasicEvent {
	/**
	 * @var int
	 */
	protected $user_id;

	public function get_group_slug() {
		return EventFactory::GROUP_USERS;
	}

	/**
	 * Returns the user objects, associated with an event
	 *
	 * @return \WP_User
	 */
	private function get_user() {
		return new \WP_User( $this->user_id );
	}

	/**
	 * @inheritDoc
	 */
	public function jsonSerialize() {
		return array_merge( parent::jsonSerialize(), [
			'user_id' => $this->user_id
		] );
	}

	/**
	 * @inheritDoc
	 */
	public function set_from_json( array $serializedJson ) {
		parent::set_from_json( $serializedJson );
		$this->user_id = $serializedJson['user_id'];

		if ( ! get_user_by( 'id', $this->user_id ) instanceof \WP_User ) {
			throw new ReferenceNoLongerAvailableException( __( "User {$serializedJson['user_id']} no longer exists.", 'shopmagic-for-woocommerce' ) );
		}
	}

	/**
	 * @inheritDoc
	 */
	public function get_provided_data_domains() {
		return [ \WP_User::class ];
	}

	/**
	 * @inheritDoc
	 */
	public function get_provided_data() {
		return [ \WP_User::class => $this->get_user() ];
	}
}
