<?php

namespace WPDesk\ShopMagic\Filter\ComparisionType;

final class DateType extends AbstractType {
	/**
	 * Is it future?
	 *
	 * @var bool
	 */
	public $has_is_future_comparision = false;

	/**
	 * Is it past?
	 *
	 * @var bool
	 */
	public $has_is_past_comparision = false;

	/**
	 * Is after specific date?
	 *
	 * @var bool
	 */
	public $has_is_after = true;

	/**
	 * Is before specific date?
	 *
	 * @var bool
	 */
	public $has_is_before = true;

	/**
	 * Is it on a specific date date?
	 *
	 * @var bool
	 */
	public $has_is_on = true;

	/**
	 * Is not on a specific date?
	 *
	 * @var bool
	 */
	public $has_is_not_on = true;

	/**
	 * Is day of the week.
	 *
	 * @var bool
	 */
	public $has_days_of_the_week = true;

	/**
	 * Is between dates.
	 *
	 * @var bool
	 */
	public $has_is_between_dates = true;

	/**
	 * Is not set?
	 *
	 * @var bool
	 */
	public $has_is_not_set = true;

	/**
	 * Is set?
	 *
	 * @var bool
	 */
	public $has_is_set = true;

	/**
	 * Our rule uses datepicker?
	 *
	 * @var bool
	 */
	public $uses_datepicker = false;

	/**
	 * @inheritDoc
	 */
	public function passed( $expected_value, $compare_type, $actual_value ) {
		// TODO: Implement passed() method.
	}

	/**
	 * @inheritDoc
	 */
	public function get_conditions() {
		$compare_types = [];

		if ( $this->has_is_future_comparision ) {
			$compare_types['is_in_the_next']     = __( 'Is in the next', 'shopmagic-for-woocommerce' );
			$compare_types['is_not_in_the_next'] = __( 'Is not in the next', 'shopmagic-for-woocommerce' );
		}

		if ( $this->has_is_past_comparision ) {
			$compare_types['is_in_the_last']     = __( 'Is in the last', 'shopmagic-for-woocommerce' );
			$compare_types['is_not_in_the_last'] = __( 'Is not in the last', 'shopmagic-for-woocommerce' );
		}

		if ( $this->has_is_after ) {
			$this->uses_datepicker     = true;
			$compare_types['is_after'] = __( 'Is after', 'shopmagic-for-woocommerce' );
		}

		if ( $this->has_is_before ) {
			$this->uses_datepicker      = true;
			$compare_types['is_before'] = __( 'Is before', 'shopmagic-for-woocommerce' );
		}

		if ( $this->has_is_on ) {
			$this->uses_datepicker  = true;
			$compare_types['is_on'] = __( 'Is on', 'shopmagic-for-woocommerce' );
		}

		if ( $this->has_is_not_on ) {
			$this->uses_datepicker       = true;
			$compare_types ['is_not_on'] = __( 'Is not on', 'shopmagic-for-woocommerce' );
		}

		if ( $this->has_days_of_the_week ) {
			$compare_types['days_of_the_week'] = __( 'Is on day/s of the week', 'shopmagic-for-woocommerce' );
		}

		if ( $this->has_is_between_dates ) {
			$compare_types['is_between'] = __( 'Is in the range', 'shopmagic-for-woocommerce' );
		}

		if ( $this->has_is_not_set ) {
			$compare_types['is_not_set'] = __( 'Is not set', 'shopmagic-for-woocommerce' );
		}

		if ( $this->has_is_set ) {
			$compare_types['is_set'] = __( 'Is set', 'shopmagic-for-woocommerce' );
		}

		return $compare_types;
	}
}
